#fileImageDisplayArea img {
    width: 100%;
    height: 100%;
    max-height: 600px;
}

#fileTextDisplayArea {
    font-family: monaco, Consolas, "Lucida Console", monospace;
    font-size: 14px;
    font-style: normal;
    font-variant: normal;
    font-weight: 400;
    line-height: 20px;
    padding: 10px;
    max-height: 600px;
}

.alert-preview {
    margin-bottom: 0px;
}